from base_class.base import base

class client_aws_sso(base): 
  """This is a library to help with the interaction with the cloud providers"""


  def __init__(self, *args, **kwargs) -> None: 
    super().__init__(*args, **kwargs)
  
  